# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
# 
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is Komodo code.
# 
# The Initial Developer of the Original Code is ActiveState Software Inc.
# Portions created by ActiveState Software Inc are Copyright (C) 2000-2007
# ActiveState Software Inc. All Rights Reserved.
# 
# Contributor(s):
#   ActiveState Software Inc
# 
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
# 
# ***** END LICENSE BLOCK *****

"""Komodo's sitepyxpcom: loaded by the PyXPCOM Component Loader
(pyloader) during XPCOM initialization.

Redirect Python's stdout/stderr to log files in Komodo user app data
dir. This is necessary on Windows if running without a console
(subsystem:windows). Otherwise Python output during PyXPCOM registration
can choke, fail, result in borked PyXPCOM and result in faulty Komodo
startup.

As well, these log files might be useful for debugging.

Note: this is only done for non-verbose mode. I.e. if "-v" is used the
output is written to the console.
"""

import os
import sys


def redirect_std_handles():
    stdout_log_name = "pystdout.log"
    stderr_log_name = "pystderr.log"

    log_dir = os.environ.get("_KOMODO_HOSTUSERDATADIR", None)
    if log_dir is not None:
        stdout_log_path = os.path.join(log_dir, stdout_log_name)
        stderr_log_path = os.path.join(log_dir, stderr_log_name)
        sys.stdout = open(stdout_log_path, "w")
        sys.stderr = open(stderr_log_path, "w")
    else:
        # Fallback to "writing" to /dev/null.
        class NullWriter:
            def __init__(self, name):
                self.name = name
            def write(self, s): pass
            def writelines(self, s): pass
            def flush(self): pass
            def close(self):
                self.closed = True
        sys.stdout = NullWriter("<stdout>")
        sys.stderr = NullWriter("<stderr>")


#---- mainline

if not os.environ.has_key("KOMODO_VERBOSE"):
    redirect_std_handles()

